local MAJOR = "StarLibWidgetTimer-1.0" 
local MINOR = 8

assert(LibStub, MAJOR.." requires LibStub") 
local WidgetTimer = LibStub:NewLibrary(MAJOR, MINOR)
if not WidgetTimer then return end
local LibTimer = LibStub("StarLibTimer-1.0", true)
assert(LibTimer, MAJOR .. " requires StarLibTimer-1.0")
local Evaluator = LibStub("StarLibEvaluator-1.0", true)
assert(Evaluator, MAJOR .. " requires StarLibEvaluator-1.0")

local pool = setmetatable({}, {__mode = "k"})

WidgetTimer.defaults = {
	expression = 'return',
	repeating = false,
	update = 0,
}

local widgetType = {timer=true}

-- Create a new StarLibWidgetTimer object
-- @name WidgetTimer.New
-- @usage WidgetTimer:New(visitor, name, config, errorLevel, callback, data)
-- @param visitor An StarLibCore-1.0 object or your own core
-- @param 
-- @param config This timeer's parameters
-- @param errorLevel The errorLevel for this object
-- @param callback Timer callback
-- @param data Some data to pass to the callback
-- @return A new StarLibWidgetTimer object
function WidgetTimer:New(visitor, name, config, errorLevel, callback, data) 
	assert(name, "WidgetTimer requires a name.")
	assert(config, "Please provide the timer with a config")
	assert(config.expression, name .. ": Please provide the timer with an expression")
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
		obj.options = {}
	end
		
	setmetatable(obj, self)

	obj.widget = LibWidget:New(obj, visitor, name, config, 0, 0, 0, widgetType, errorLevel)
	
	obj.callback = callback
	obj.data = data
	obj.expression = config.expression
	obj.repeating = config.repeating
		
	obj.timer = LibTimer:New("WidgetTimer.timer " .. self.name, config.update or defaults.update, config.repeating, self.Update, self)
	
	return obj	
end


-- Delete a StarLibWidgetTimer object
-- @name StarLibWidgetTimer.Del
-- @usage StarLibWidgetTimer:Del([marq])
-- @param marq A StarLibWidgetTimer object
function WidgetTimer:Del(timer)
	if not timer then
		timer = self
	end
	timer:Stop()
	timer.widget:Del()
	pool[timer] = true
end

-- Start a StarLibWidgetTimer
-- @name StarLibWidgetTimer.Start
-- @usage object:Start()
-- @return Nothing
function WidgetTimer:Start()
	self.error:Print("WidgetTimer:Start")
	self.oldBuffer = nil
	self:Update()
	if self.timer then
		self.timer:Start()
	end
	if self.textTimer then
		self.textTimer:Start()
	end
end

-- Stop a StarLibWidgetTimer
-- @name StarLibWidgetTimer.Stop
-- @usage object:Stop()
-- @return Nothing
function WidgetTimer:Stop()
	self.error:Print("WidgetTimer:Stop")
	if self.timer then 
		self.timer:Stop()
	end
	if self.textTimer then
		self.textTimer:Stop()
	end
	self.oldBuffer = nil
end

-- Update data
-- @name StarLibWidgetTimer.Update
-- @name StarLibWidgetTimer.Update
-- @usage StarLibWidgetTimer:Update()
-- @return True if we updated.
function WidgetTimer:Update()
	Evaluator.ExecuteCode(self.visitor.environment, self.name, self.expression)
end

function WidgetTimer:GetOptions(visitor, db)
		local defaults = WidgetTimer.defaults
		local options = {
			update = {
				name = "Update Rate",
				desc = "Enter the timer's refresh rate",
				type = "input",
				pattern = "%d",
				get = function()
					return tostring(db.update or defaults.update)
				end,
				set = function(info, v)
					db.update = tonumber(v)
				end,
				order = 2
			},
			repeating = {
				name = "Repeating",
				desc = "Toggle whether to repeat this timer",
				type = "toggle",
				get = function()
					return db.repeating or defaults.repeating
				end,
				set = function(info, v)
					db.repeating = v
				end,
				order = 1
			},
			expression = {
				name = "Expression",
				desc = "Enter this widget's expression",
				type = "input",
				width = "full",
				multiline = true,
				get = function()
					return db.expression or defaults.expression
				end,
				set = function(info, v)
					db.expression = v
				end,
				order = 3
			},
			
		}
	return options
end
